@@ -159,7 +159,6 @@ mate_panel_applets_manager_get_applet_factory_info_from_file (const gchar *filen
 {
 	MatePanelAppletFactoryInfo *info;
 	GKeyFile               *applet_file;
-	const char             *lib_prefix;
 	gchar                 **groups;
 	gsize                   n_groups;
 	gsize                   i;
@@ -189,6 +188,8 @@ mate_panel_applets_manager_get_applet_factory_info_from_file (const gchar *filen
 	info->in_process = g_key_file_get_boolean (applet_file, MATE_PANEL_APPLET_FACTORY_GROUP,
 						   "InProcess", NULL);
 	if (info->in_process) {
+		const char *lib_prefix;
+
 		info->location = g_key_file_get_string (applet_file, MATE_PANEL_APPLET_FACTORY_GROUP,
 							"Location", NULL);
 		if (!info->location) {
@@ -335,15 +336,15 @@ applets_directory_changed (GFileMonitor     *monitor,
 static void
 mate_panel_applets_manager_dbus_load_applet_infos (MatePanelAppletsManagerDBus *manager)
 {
-	GSList      *dirs, *d;
-	GDir        *dir;
-	const gchar *dirent;
-	GError      *error = NULL;
+	GSList *dirs;
 
 	dirs = mate_panel_applets_manager_get_applets_dirs ();
-	for (d = dirs; d; d = g_slist_next (d)) {
+	for (GSList *d = dirs; d; d = g_slist_next (d)) {
+		GDir         *dir;
+		const gchar  *dirent;
 		GFileMonitor *monitor;
 		GFile        *dir_file;
+		GError       *error = NULL;
 		gchar        *path = (gchar *) d->data;
 
 		dir = g_dir_open (path, 0, &error);
